**************************************************************************
**This is the program written for SAS version6.07 that was used to      **
**generate the starting values for Michael Dueker's July/August 1993    **
**Review article.  Note:  This version of the program computes the      **
**starting values for Germany and the United Kingdom.  If you wanted to **
**obtain the starting values for a different set of countries, say Japan** 
**and the U.S., then you would just substitute the pneumonics for these **
**two countries into the program (in this case, EXJPUS, JAPCPI, and     **
**USCPI).                                                               **
**************************************************************************
;

%MACRO PHILHANS(DATASET=,DEP=,INDEP=,PSILAGS=,TESTROOT=,ADFLAGS=);

PROC IML;

USE &DATASET;
READ ALL VAR{&DEP} INTO Y;

USE &DATASET;
READ ALL VAR{&INDEP} INTO X1;

NUMOBS = NROW(Y);
NVAR   = NCOL(X1);
N      = NUMOBS-1;
ONES   = J(NUMOBS,1,1);

X = ONESX1;

NVARINT = NCOL(X);

XPXI = INV(X`*X);
BETA = XPXI*X`*Y;
RES = Y - X*BETA;

***********************************************
** TO GET DIAGNOSTICS FOR OLS AND PRINT THEM **
***********************************************;

SSE = SSQ(RES);
DFE = NUMOBS - NVARINT;
MSE = SSE/DFE;
CSSY = SSQ(Y - SUM(Y)/NUMOBS);
RSQUARE = (CSSY - SSE)/CSSY;

PRINT,"OUTPUT FOR &DEP = INTERCEPT + &INDEP, USING &DATASET";
PRINT,"OLS REGRESSION RESULTS",
     SSE '   ' DFE '   ' MSE '   ' RSQUARE,;

STDB = SQRT(VECDIAG(XPXI)*MSE);
T = BETA/STDB;
PROB = 1 - PROBF(T#T,1,DFE);

PRINT,"OLS PARAMETER ESTIMATES",
     BETA '   ' STDB '   ' T '   ' PROB,;

***************************************************
** START OF CODE TO DO PHILLIPS-HANSEN ESTIMATOR **
***************************************************;

V = X1(2:NUMOBS,) - X1(1:N,);
U = RES(2:NUMOBS,1);
PSI = UV;
COV = PSI`*PSI/N;

LAM = PSI(2:N,)` * PSI(1:N-1,) / (N-1);

DO I = 2 TO &PSILAGS;
 LAM = ( PSI((I+1):N,)` * PSI(1:(N-I),) / (N-I) ) + LAM;
END;

OMEGA = COV + LAM + LAM`;
NCOLMEGA = NCOL(OMEGA);

MEG22 = OMEGA (2:NCOLMEGA,2:NCOLMEGA);
MEG22INV = INV(MEG22);

MEG12 = (OMEGA(1,2:NCOLMEGA));
MEG21 = (OMEGA(2:NCOLMEGA,1));
MEG11 = (OMEGA(1,1));

MEGADD = MEG11 - MEG12*MEG22INV*MEG21;

COR1 = (OMEGA(1,2:NCOLMEGA))*MEG22INV;

Y1 = Y(2:NUMOBS,)-V*COR1`;
XPXI2 = INV( X(2:NUMOBS,)`*X(2:NUMOBS,) );
BETA1 = XPXI2 * X(2:NUMOBS,)` * Y1;

A = COV(2:NCOLMEGA,1) + LAM(2:NCOLMEGA,1) -
   (COV(2:NCOLMEGA,2:NCOLMEGA) + LAM(2:NCOLMEGA,2:NCOLMEGA))*COR1`;
ANEW = (0A`);
A1 = N*ANEW;

BETA2 = BETA1-XPXI2*A1`;
RES2 = Y -  X * BETA2;

SSE2 = SSQ(RES2);
DFE2 = NROW(Y1) - NCOL(X);
MSE2 = SSE2/DFE2;
CSSY2 = SSQ( Y(2:NUMOBS,) - SUM( Y(2:NUMOBS,) )/N);
RSQUARE2 = (CSSY2 - SSE2)/CSSY2;
PRINT,"PHILLIPS HANSEN CORRECTED REGRESSION RESULTS",
     SSE2 '   ' DFE2 '   ' MSE2 '   ' RSQUARE2;
STDB2 = SQRT(VECDIAG(XPXI2)*MEGADD);
T2 = BETA2/STDB2;
PROB2 = 1 - PROBF(T2#T2,1,DFE);
PRINT,"FULLY MODIFIED PARAMETER ESTIMATES",
 BETA2 '   ' STDB2 '   ' T2 '   ' PROB2,;

*****************************************************
** TO DO AUGMENTED DICKEY-FULLER TEST WITH ADFLAGS **
**               AND PRINT RESULTS.                **
*****************************************************;

%MACRO ADF(RES2=,ADFLAGS=,NUMOBS=);

NUMOBS = &NUMOBS;
ADFLAGS = &ADFLAGS;

BASEMATX = J(NUMOBS , ADFLAGS + 2, 0 );
BASEMATX( ,1) = &RES2;

DO I = 2 TO NUMOBS;
 DO J = 2 TO NCOL(BASEMATX);
  BASEMATX(I,J) = BASEMATX(I-1,J-1);
 END;
END;

LAGLEVEL = BASEMATX( ADFLAGS+2: NUMOBS, );
LAGCHANG = J( NROW(LAGLEVEL),NCOL(LAGLEVEL) - 1, 0);

DO I = 1 TO NROW(LAGLEVEL);
 DO J = 1 TO ( NCOL(LAGLEVEL) - 1 );
  LAGCHANG(I,J) = LAGLEVEL(I,J) - LAGLEVEL(I,J+1);
 END;
END;

DFY = LAGCHANG( ,1);
DFX = J( NROW(LAGLEVEL) , 1, 1 )  LAGLEVEL( ,2) 
      LAGCHANG( ,2:NCOL(LAGCHANG) );

DF_N = NROW(DFX);
DFXPDFX = DFX` * DFX;

DF_BETA = ( (INV(DFXPDFX))*DFX` ) * DFY;
DF_RES = DFY - DFX*DF_BETA;
DF_SSE = SSQ(DF_RES);
DF_DFE = NROW(DFX) - NCOL(DFX);
DF_MSE = DF_SSE/DF_DFE;
DF_STDB = SQRT( VECDIAG( INV(DFXPDFX) ) * DF_MSE );
DF_T = DF_BETA/DF_STDB;
DF_PROB = 1 - PROBF(DF_T#DF_T,1,DF_DFE);
NAMES={'INTERCEPT','1 LAG LEVEL'};
CHGNAME = J(ADFLAGS,1,'TEMPORARY      ');

%DO I = 1 %TO &ADFLAGS;
 CHGNAME(&I,1) = "&I LAGS CHANGE";
%END;

VARNAMES = NAMES // CHGNAME;

PRINT,"AUGMENTED DICKEY-FULLER UNIT ROOT TEST RESULTS" ' ' DF_N,
   VARNAMES ' ' DF_BETA '   ' DF_STDB '   ' DF_T '   ' DF_PROB,,,,;

%MEND ADF;


***************************************************************
**  TO DO 'REGULAR' DICKEY-FULLER TEST, REGRESSING THE RESID **
**     ON AN INTERCEPT, THE LAG LEVEL, AND A TIME TREND.     **
***************************************************************;

%MACRO DF(RES2=,NUMOBS=);
 NUMOBS=&NUMOBS;
 TIME = 1:(NUMOBS-1);
 DFY = &RES2(2:NUMOBS, );
 DFX = J(NUMOBS-1,1,1)  &RES2(1:NUMOBS-1, )  TIME`;
 DFXPDFX = DFX` * DFX;
 DF_N = NROW(DFX);

DF_BETA = ( (INV(DFXPDFX))*DFX` ) * DFY;
DF_RES = DFY - DFX*DF_BETA;
DF_SSE = SSQ(DF_RES);
DF_DFE = NROW(DFX) - NCOL(DFX);
DF_MSE = DF_SSE/DF_DFE;
DF_STDB = SQRT( VECDIAG( INV(DFXPDFX) ) * DF_MSE );
DF_T = DF_BETA/DF_STDB;
DF_PROB = 1 - PROBF(DF_T#DF_T,1,DF_DFE);

VARNAMES={'INTERCEPT','1 LAG LEVEL','TIME TREND'};

PRINT,"DICKEY-FULLER UNIT ROOT TEST RESULTS" ' ' DF_N,
   VARNAMES ' ' DF_BETA '   ' DF_STDB '   ' DF_T '   ' DF_PROB,,,,;

%MEND DF;

IF &TESTROOT = 'ADF' THEN DO;
 %ADF(RES2=RES2,ADFLAGS=&ADFLAGS,NUMOBS=NUMOBS) END;


IF &TESTROOT = 'DF'  THEN DO;
 %DF(RES2=RES2,NUMOBS=NUMOBS) END;


%MEND PHILHANS;

FILENAME IN 'H1CAW01.DUEKER.CNTL';

DATA A;
     INFILE IN(EXCHANGE) FIRSTOBS=13;
     INFORMAT DATE MMDDYY10.;
     INPUT DATE EXUKUS EXFRUS EXGEUS EXJPUS;
     EXJPGE = EXJPUS/EXGEUS ;
     EXJPUK = EXJPUS/EXUKUS ;
     EXGEUK = EXGEUS/EXUKUS ;

DATA B;
     INFILE IN(CPI) FIRSTOBS=11;
     INFORMAT DATE MMDDYY10.;
     INPUT DATE USCPI UKCPI GERCPI JAPCPI FRACPI;

DATA C; MERGE A(KEEP=DATE EXGEUK) B(KEEP=DATE GERCPI UKCPI); BY DATE;
     IF DATE GE '15NOV92'D THEN DELETE;

     EXGEUK = LOG(EXGEUK);
     UKCPI   = LOG(UKCPI);
     GERCPI  = LOG(GERCPI);

%PHILHANS(DATASET=C ,DEP=EXGEUK ,INDEP=GERCPI UKCPI ,PSILAGS=30,
          TESTROOT='ADF',ADFLAGS=4 );
